module Feature.Sharing.SQLite where

import Core.Types
import Core.SQLite
import Feature.Sharing.Types

import Control.Monad.Reader           (ask, lift, liftIO)
import Crypto.Simple.CBC              (encrypt, decrypt)
import Data.Maybe                     (listToMaybe)
import Data.Time.Clock.POSIX          (getPOSIXTime)
import Database.SQLite.Simple

import qualified Data.ByteString.Base64 as B64
import qualified Data.ByteString.Char8  as B
import qualified Data.Text              as T
import qualified Data.Text.Encoding     as ET
import qualified Data.Text.Lazy         as LT

findByLink :: String -> PurrAction (Maybe T.Text)
findByLink link = do
  db    <- dbPath
  key   <- encKey
  conn  <- liftIO $ open db
  res   <- liftIO $ query conn "SELECT * from pws WHERE link = ?" (Only link)
  liftIO $ close conn
  readEncryptedSecret key res

insertNewSecret :: T.Text -> Integer -> T.Text -> Integer -> PurrAction ()
insertNewSecret sec life link maxViews = do
  db     <- dbPath
  key    <- encKey
  encSec <- liftIO $ encryptSecret key sec
  conn   <- liftIO $ open db
  time   <- liftIO $ epochTime
  liftIO  $ execute conn 
    "INSERT INTO pws (link, secret, date, life, views, maxViews) VALUES (?, ?, ?, ?, ?, ?)"
      (SecretEntry link (encodeSecret encSec) time life 0 maxViews)
  liftIO  $ close conn

readEncryptedSecret :: String -> [SecretEntry] -> PurrAction (Maybe T.Text)
readEncryptedSecret key sec = do
  decKey <- liftIO
    (sequence $ decryptSecret key <$> decodeSecret <$> listToMaybe sec)
  return (ET.decodeLatin1 <$> decKey)

encodeSecret :: B.ByteString -> T.Text
encodeSecret b = ET.decodeUtf8 $ B64.encode b

decodeSecret :: SecretEntry -> B.ByteString
decodeSecret s = B64.decodeLenient $ ET.encodeUtf8 (secret s)

encryptSecret :: String -> T.Text -> IO B.ByteString
encryptSecret k s = encrypt (B.pack k) (ET.encodeUtf8 s)

decryptSecret :: String -> B.ByteString -> IO B.ByteString
decryptSecret k b = decrypt (B.pack k) b 

epochTime :: IO Integer
epochTime = fmap round getPOSIXTime
